//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision highp float;

varying vec2 TexCoord;
uniform vec4 RTTPixelOffset;
uniform sampler2D Tex;

vec4 GaussianBlur7(sampler2D tex, vec2 baseUV, vec2 pixelOffset)
{
  vec4 ret = vec4(0,0,0,0);
  const int n = 2;
  float gWeights[n];
  gWeights[0] = 0.44908;
  gWeights[1] = 0.05092;
  float gOffsets[n];
  gOffsets[0] = 0.53805;
  gOffsets[1] = 2.06278;
  for(int i = 0; i < n; ++i)
  {
    vec2 texCoordOffset = gOffsets[i] * pixelOffset;
    vec4 col = texture2D(tex, baseUV + texCoordOffset) + texture2D(tex, baseUV - texCoordOffset);
    ret += gWeights[i] * col;
  }
  return ret;
}

vec4 GaussianBlur15(sampler2D tex, vec2 baseUV, vec2 pixelOffset)
{
  vec4 ret = vec4(0,0,0,0);
  const int n = 4;
  float gWeights[n];
  gWeights[0] = 0.24961;
  gWeights[1] = 0.19246;
  gWeights[2] = 0.05148;
  gWeights[3] = 0.00645;
  float gOffsets[n];
  gOffsets[0] = 0.64434;
  gOffsets[1] = 2.37885;
  gOffsets[2] = 4.29111;
  gOffsets[3] = 6.21661;
  for(int i = 0; i < n; ++i)
  {
    vec2 texCoordOffset = gOffsets[i] * pixelOffset;
    vec4 col = texture2D(tex, baseUV + texCoordOffset) + texture2D(tex, baseUV - texCoordOffset);
    ret += gWeights[i] * col;
  }
  return ret;
}


vec4 GaussianBlur35(sampler2D tex, vec2 baseUV, vec2 pixelOffset)
{
  vec4 ret = vec4(0,0,0,0);
  const int n = 9;
  float gWeights[n];
  gWeights[0] = 0.10855;
  gWeights[1] = 0.13135;
  gWeights[2] = 0.10406;
  gWeights[3] = 0.07216;
  gWeights[4] = 0.04380;
  gWeights[5] = 0.02328;
  gWeights[6] = 0.01083;
  gWeights[7] = 0.00441;
  gWeights[8] = 0.00157;
  float gOffsets[n];
  gOffsets[0] = 0.66293;
  gOffsets[1] = 2.47904;
  gOffsets[2] = 4.46232;
  gOffsets[3] = 6.44568;
  gOffsets[4] = 8.42917;
  gOffsets[5] = 10.41281;
  gOffsets[6] = 12.39664;
  gOffsets[7] = 14.38070;
  gOffsets[8] = 16.36501;
  for(int i = 0; i < n; ++i)
  {
    vec2 texCoordOffset = gOffsets[i] * pixelOffset;
    vec4 col = texture2D(tex, baseUV + texCoordOffset) + texture2D(tex, baseUV - texCoordOffset);
    ret += gWeights[i] * col;
  }
  return ret;
}
vec4 test(sampler2D tex, vec2 baseUV, vec2 PixelOffset)
{
  vec4 ret = texture2D(tex, baseUV);
  return ret;
}

void main()
{
  gl_FragColor = GaussianBlur15(Tex, TexCoord, vec2(RTTPixelOffset.x, RTTPixelOffset.y));
}