//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision mediump float;

varying vec2 TexCoord;

struct MaterialInfo
{
  vec3 Ka;            // light blend color
  vec3 Kd;            // dark blend color
  float Shininess;    // the alpha value
};
uniform MaterialInfo Material;

void computePatternColor(vec2 uv, vec4 color1, vec4 color2, float radius, out vec4 outColor)
{
  vec2 nuv = fract(uv);
  
  vec2 centers[4];
  centers[0] = vec2(0.0, 0.5);
  centers[1] = vec2(0.5, 0.0);
  centers[2] = vec2(0.5, 1.0);
  centers[3] = vec2(1.0, 0.5); // Centers for the dots.
  
  for(int i = 0; i < 4; ++i)
  {
    if(distance(nuv, centers[i]) < radius)
    {
      outColor = vec4(color1.rgb, 1.0);
      return;
    }
  }
  
  if(nuv.y < 0.5 - nuv.x)
  {
    nuv.x += 0.5;
    nuv.y += 0.5;
  }
  else if(nuv.y > nuv.x + 0.5)
  {
    nuv.x += 0.5;
    nuv.y -= 0.5;
  }
  else if(nuv.y > 1.5 - nuv.x)
  {
    nuv.x -= 0.5;
    nuv.y -= 0.5;
  }
  else if(nuv.y < nuv.x - 0.5)
  {
    nuv.x -= 0.5;
    nuv.y += 0.5;
  }
  outColor = mix(color1, color2, nuv.y);
}

void main(void)
{
  float intensity = 1.0;
  float tiles = 0.024;
  vec2 uv = TexCoord * tiles;
  // Times 1.5 so the fringe is larger and we won't see the boundary.
  //vec2 r = vec2(pow((TexCoord.x) * 1.5, 2.0), pow((TexCoord.y) * 1.5, 2.0));
  //intensity *= max(0.0, 0.58 - pow(min(1.0, 2.5 * dot(r, r)), 2.0));
  lowp vec4 texColor;
  vec4 color1 = vec4(Material.Ka, Material.Shininess);
  vec4 color2 = vec4(Material.Kd, Material.Shininess);
  computePatternColor(uv, color1, color2, 0.07, texColor);
  gl_FragColor = texColor;
}
