//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

attribute vec3 VertexPosition;
attribute vec2 VertexTexCoord;

uniform float aspectHW;
uniform float scaleFactor;       // To compensate the image size change due to rotation on iPad.
uniform mat4 MVP;               // Projection * Model * View
uniform mat4 ViewMatrix;
uniform vec2 anchor;            // Provide way to set different anchor dynamically.

varying vec2 TexCoord;

mat3 getRotationMat(vec3 from, vec3 to) // Please normalize from and to.
{
  vec3 x = normalize(cross(from, to));
  float theta = acos(dot(from, to));
  mat3 R = mat3(1,0,0,0,1,0,0,0,1);// Given in the column order.
  if(theta < 1e-1) {
    return R;
  }
  mat3 A = mat3(0.0, x.z, -x.y, -x.z, 0.0, x.x, x.y, -x.x, 0.0);
  
  R += sin(theta) * A  + (1.0 - cos(theta)) * A * A;
  return R;
}

void main()
{
  TexCoord = vec2(VertexTexCoord.x, 1.0 - VertexTexCoord.y);
  vec4 originPoint = vec4(0, 0, 0, 1);
  // Calculate the updated vertex position.
  gl_Position = MVP * originPoint;
  gl_Position /= gl_Position.w;
  
  // obtain the vector of up in camera space into the world space.
  vec3 y = vec3(0, 1, 0);
  vec3 z = vec3(0, 0, 1);
  vec3 up = normalize(vec3(ViewMatrix[1][0], ViewMatrix[1][1], ViewMatrix[1][2]));
  up = normalize(up - dot(up, z) * z);
  mat3 R = getRotationMat(up, y);
  
  vec2 adjustVertexPosition = vec2(R[0][0] * VertexPosition.x + R[0][1] * VertexPosition.y, 
                                   R[1][0] * VertexPosition.x + R[1][1] * VertexPosition.y);
  
  gl_Position.xy += vec2(adjustVertexPosition.x + anchor.x, (adjustVertexPosition.y + anchor.y) / aspectHW) * scaleFactor;
  gl_Position.z   = -0.99; // Make sure this is always drawn on screen in a close range of the absolute front.  
  

}
