//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

attribute vec3 VertexPosition;
attribute vec3 VertexNormal; // special coded pre and next 3D vertex.
attribute vec2 VertexTexCoord; // direction in first value.

varying vec2 TexCoord;

uniform mat4 MVP;
uniform float lineWid;
uniform float aspect;
float miterLimit = 2.0;

void main()
{
  vec2 aspectVec = vec2(aspect, 1.0);
  vec3 pre, next;
  if(VertexNormal.z == 0.0) {
    pre = VertexPosition + vec3(VertexNormal.x, 0.0, 0.0);
    next = VertexPosition + vec3(0.0, VertexNormal.y, 0.0);
  }
  else
  {
    next = VertexPosition + vec3(VertexNormal.x, 0.0, 0.0);
    pre = VertexPosition + vec3(0.0, VertexNormal.y, 0.0);
  }
  vec4 prevProj = MVP * vec4(pre, 1.0);
  vec4 nextProj = MVP * vec4(next, 1.0);
  vec4 curProj = MVP * vec4(VertexPosition, 1.0);
  
  // Obtain 2D sceen space coords.
  vec2 curScr = curProj.xy / curProj.w * aspectVec;
  vec2 prevScr = prevProj.xy / prevProj.w * aspectVec;
  vec2 nextScr = nextProj.xy / nextProj.w * aspectVec;
  
  float len = lineWid;
  float orientation = VertexTexCoord.x;
  
  // starting point uses (cur - previous)
  vec2 dir = vec2(0, 0);
  if(curScr == prevScr) {
    dir = normalize(nextScr - curScr);
  }
  else if(curScr == nextScr){ // end point.
    dir = normalize(curScr - prevScr);
  }
  else{
    // it has both prev and next, the direction needs combination.
    vec2 dirA = normalize(curScr - prevScr);
    vec2 dirB = normalize(nextScr - curScr);
    vec2 tangent = normalize(dirA + dirB);
    vec2 perp = vec2(-dirA.y, dirA.x);
    vec2 miter = vec2(-tangent.y, tangent.x);
    float miterDot = dot(miter, perp);
    len = miterDot == 0.0 ? 0.0 : lineWid / miterDot;
    len = clamp(len, 0.0, lineWid * miterLimit);
    dir = tangent;
  }
  vec2 normal = vec2(-dir.y, dir.x);
  normal *= len / 2.0;
  normal.x /= aspect;
  vec4 offset = vec4(normal * orientation, 0.0, 0.0);
  
  TexCoord = vec2(mod(VertexTexCoord.y, 2.0), (1.0 + VertexTexCoord.x) / 2.0);
  
  gl_Position = curProj + offset;
}