//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision highp float;

varying vec2 interp_depth;

float unpack(vec4 comp)
{
  const vec4 bitSh = vec4(
     1.0,
     1.0/256.0,
     1.0/(256.0*256.0),
     1.0/(256.0*256.0*256.0)
  );

  return dot(bitSh,comp);
}

vec4 pack (float depth)
{
    const vec4 bitSh = vec4(256.0 * 256.0 * 256.0,
                            256.0 * 256.0,
                            256.0,
                            1.0);
    const vec4 bitMsk = vec4(0,
                             1.0 / 256.0,
                             1.0 / 256.0,
                             1.0 / 256.0);
    vec4 comp = fract(depth * bitSh);
    comp -= comp.xxyz * bitMsk;
    return comp;
}

vec4 EncodeFloatRGBA( float v ) {
  vec4 enc = vec4(1.0, 255.0, 65025.0, 16581375.0) * v;
  enc = fract(enc);
  enc -= enc.yzww * vec4(1.0/255.0,1.0/255.0,1.0/255.0,0.0);
  return enc;
}
float DecodeFloatRGBA( vec4 rgba ) {
  return dot( rgba, vec4(1.0, 1.0/255.0, 1.0/65025.0, 1.0/16581375.0) );
}

void main()
{
  // this gives a value between -1 and 1, with -1 being closest.
  highp float depth =  interp_depth.x / interp_depth.y;
  depth = (depth + 1.0) / 2.0; // convert to 0 to 1.
  //vec4 packed_depth = pack(depth);
  vec4 packed_depth = EncodeFloatRGBA(depth);
  gl_FragColor = packed_depth;
}

