//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision mediump float;

varying vec3 Position;
varying vec3 Normal;

struct LightInfo
{
  vec4 Position;    // Light position/direction in eye coords.
  vec3 La;          // Ambient intensity
  vec3 Ld;          // Diffuse intensity
  vec3 Ls;          // Specular intensity
  vec3 Ld_Sky;      // Sky color
  vec3 Ld_Ground;   // Ground color
};
uniform LightInfo Light;

struct MaterialInfo
{
  vec3 Ka;            // Ambient reflectivity
  vec3 Kd;            // Diffuse reflectivity
  vec3 Ks;            // Specular reflectivity
  float Shininess;    // Specular shininess factor
};
uniform MaterialInfo Material;

void phongModel(vec3 pos, vec3 norm, out vec3 ambAndDiff, out vec3 spec)
{
  vec3 s = normalize(vec3(-Light.Position));
  float sDotN = max(dot(s, norm), 0.0);
  ambAndDiff = Light.La * Material.Ka + Material.Kd * Light.Ld * sDotN;
  vec3 v = vec3(0, 0, 1);   // for non-local viewer
  // vec3 v = normalize(-pos.xyz);
  // use halfway vector to save computation
  vec3 h = normalize(v + s);
  spec = Light.Ls * Material.Ks * pow(max(dot(h, norm), 0.0), Material.Shininess);
}

void main()
{
  vec3 ambAndDiff, spec;
  phongModel(Position, Normal, ambAndDiff, spec);
  gl_FragColor = vec4(ambAndDiff + spec, 1.0);
}