//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision mediump float;

varying vec2 TexCoord;

#ifndef USE_TEXTURE

uniform vec4 lightColor;            // light blend color
uniform vec4 darkColor;             // dark blend color

#else

uniform lowp sampler2D Tex0;            // movement color
uniform lowp sampler2D Tex1;            // rotation color
uniform float useRotationUI;             // 1.0 then we use the rotation UI

#endif

uniform vec2 scaleFactor;
uniform float fadeIn;
uniform float opacity;
  
#ifndef USE_TEXTURE
void computePatternColor(vec2 uv, vec4 color1, vec4 color2, float radius, out vec4 outColor)
{
  vec2 nuv = fract(uv);
  
  vec2 centers[4];
  centers[0] = vec2(0.0, 0.5);
  centers[1] = vec2(0.5, 0.0);
  centers[2] = vec2(0.5, 1.0);
  centers[3] = vec2(1.0, 0.5); // Centers for the dots.
  
  for(int i = 0; i < 4; ++i)
  {
    if(distance(nuv, centers[i]) < radius)
    {
      outColor = vec4(color1.rgb, 1.0);
      return;
    }
  }
  
  if(nuv.y < 0.5 - nuv.x)
  {
    nuv.x += 0.5;
    nuv.y += 0.5;
  }
  else if(nuv.y > nuv.x + 0.5)
  {
    nuv.x += 0.5;
    nuv.y -= 0.5;
  }
  else if(nuv.y > 1.5 - nuv.x)
  {
    nuv.x -= 0.5;
    nuv.y -= 0.5;
  }
  else if(nuv.y < nuv.x - 0.5)
  {
    nuv.x -= 0.5;
    nuv.y += 0.5;
  }
  outColor = mix(color1, color2, nuv.y);
}
#else
void computeUIColor(vec2 uv, out vec4 outColor)
{
  if(useRotationUI > 0.5)
  {
    outColor = texture2D(Tex1, uv);
  }
  else
  {
    outColor = texture2D(Tex0, uv);
  }
}
#endif

float computeColorAlpha(float opacity, float isFadeIn)
{
  float stepVar = step(0.5, isFadeIn); // Avoid branch in glsl.
  float leftAlpha = mix(1.0, 0.0, stepVar);
  float rightAlpha = 1.0 - leftAlpha;
  float percent = clamp(opacity, 0.0, 1.0);
  return mix(leftAlpha, rightAlpha, percent);
}

void main(void)
{
  float tiles = 0.01;
  vec2 uv = TexCoord * tiles;
  uv.x *= scaleFactor.x;
  uv.y *= scaleFactor.y;
  lowp vec4 texColor;
  vec4 lc = vec4(1.0, 0.0, 0.0, 1.0);
  vec4 dc = vec4(0.7, 0.0, 0.0, 1.0);
#ifndef USE_TEXTURE
  computePatternColor(uv, lightColor, darkColor, 0.05, texColor);
#else
  computeUIColor(TexCoord, texColor);
#endif
  texColor.a *= computeColorAlpha(opacity, fadeIn);
  gl_FragColor = texColor;
}
