//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision mediump float;

#ifdef USE_TEXTURE
varying vec2 TexCoord;
varying vec3 MeshScale;
uniform lowp sampler2D Tex;
uniform vec2 planeSize;
uniform float isDefaultScale;
#endif

uniform float sofarFrame;
uniform float startIntensity;
uniform float endIntensity;
uniform float totalFrame;

// This only work right when end is larger than start.
float flashIntensityUpdate(float startIntensity, float endIntensity)
{
  float intensity = 0.0;
  float halfPlay = totalFrame / 2.0;
  float curFrame = mod(sofarFrame, totalFrame);
  if(curFrame > totalFrame / 2.0)
  {
    intensity = 1.0 - (curFrame - halfPlay) / halfPlay;
  }
  else
  {
    intensity = curFrame / halfPlay;
  }
  float dis = max(endIntensity - startIntensity, 0.0);
  return startIntensity + intensity * dis;
}

float intensityUpdate(float startIntensity, float endIntensity)
{
  float curFrame = mod(sofarFrame, totalFrame);
  float percent = curFrame / totalFrame;

  return mix(startIntensity, endIntensity, percent);
}

void main(void)
{
  float intensity = 1.0;
  
#ifdef USE_TEXTURE
  vec2 uvDelta = vec2(1.0, 1.0);
#ifdef REPEAT_TEXTURE
  float tiles = 1.3/400.0;
  uvDelta = vec2(planeSize.x * tiles, planeSize.y * tiles); // maintain whether the size of the cursor has been changed or not.
  // Times 1.5 so the fringe is larger and we won't see the boundary.
  vec2 r = vec2(pow((TexCoord.x - 0.5) * 1.5, 2.0), pow((TexCoord.y - 0.5) * 1.5, 2.0));
  intensity *= isDefaultScale == 0.0 ? max(0.0, 1.0 - pow(min(1.0, 2.8 * dot(r, r)), 2.0)) : clamp(1.0 - 2.4 * clamp(length(vec2(TexCoord.x - 0.5, TexCoord.y - 0.5)), 0.0, 0.5), 0.0, 1.0); 
#endif
  
  vec2 uv = TexCoord * uvDelta;
  lowp vec4 texColor = texture2D(Tex, uv);

  gl_FragColor = vec4(texColor.rgb, texColor.a * intensity);
  
#else
  if(sofarFrame >= 0.0 && sofarFrame < totalFrame) // Caller wants animation.
  {
    intensity = flashIntensityUpdate(startIntensity, endIntensity);
  }
  gl_FragColor = vec4(1.0, 1.0, 0.0, intensity);
#endif

}
