//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

attribute vec3 VertexPosition;
attribute vec3 VertexNormal;
attribute vec2 VertexTexCoord;

uniform mat4 ModelMatrix;
uniform mat4 ModelViewMatrix;
uniform mat4 MVP;               // Projection * ModelView
uniform mat3 NormalMatrix;

varying vec2 TexCoord;

// Provide the scale factor for table top and ground for big and small size of plane locator.
// The goal is to scale the locator by 1 when it's on the ground, and by 1/tableFactor when it's on a table.  
// For lack of a better model, we use distance between the planeLocator and the camera to guess whether it's on the table or the ground.

// If distance < tableThreshold, we scale by 1/tableFactor.
// if distance > groundThreshold, we scale be 1.
// In-between, we interpolate using a cos shape so that it scales less than linearly close up.  It is continuous with both < tableThreshold and > groundThreshold.

//If the groundThreshold <= tableThreshold (assumed ground distance is less than assumed table distance) we give up and return a scale of 1.

float getSizeFactor(float distance, float tableThreshold, float groundThreshold, float tableFactor)
{
  if(groundThreshold <= tableThreshold) {
    return 1.0;
  }
  if(distance < tableThreshold) {
    return tableFactor;
  }
  else if(distance < groundThreshold) {
    float factor = 1.0 + (tableFactor - 1.0) * cos(3.1415926 / 2.0 * (distance - tableThreshold) / (groundThreshold - tableThreshold)); // [tableFactor to 1.0] in a cosine curve.
    return factor;
  }
  return 1.0;
}

void main()
{
  TexCoord = VertexTexCoord;

#ifdef ADJUST_SIZE_BASED_ON_DISTANCE
  // Based on the distance between the plane and the camera, we adjust the size.
  vec3 trans = vec3(ModelViewMatrix[3][0], ModelViewMatrix[3][1], ModelViewMatrix[3][2]);
  float distance = length(trans);
  vec3 adjustedPosition = VertexPosition / getSizeFactor(distance, 600.0, 1000.0, 3.2);
  gl_Position = MVP * vec4(adjustedPosition, 1.0);
#else
  gl_Position = MVP * vec4(VertexPosition, 1.0);
#endif
}
