//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

#version 100

precision highp float;

varying highp vec4 shadowMapCoord;

uniform sampler2D Tex;

vec4 VSMCompute(float distance)
{
  vec2 m = texture2D(Tex, shadowMapCoord.xy).rg;
  
  if(distance <= m.x + 0.01) return vec4(0.0, 0.0, 0.0, 0.0);
  float variance = m.y - (m.x * m.x);
  variance = max(variance, 0.01); //0.006 // Positive value, larger the lighter of the shadow.
  
  float d = distance - m.x;
  float p_max = variance / (variance + d*d); // sqrt(d) * d
  return vec4(0.2, 0.2, 0.2, clamp(1.0 - p_max, 0.0, 1.0));
}

void main(void)
{
  vec3 diffuseColor = vec3(1.0, 1.0, 1.0);
  float z = shadowMapCoord.z / shadowMapCoord.w;
  vec4 shadow_factor = VSMCompute(z);
  if(shadow_factor.w < 0.01) discard;
  gl_FragColor = vec4(shadow_factor.xyz * diffuseColor, shadow_factor.w);
}