//Wiki - https://w.amazon.com/index.php/Instant_Experience_-_Animation_Triggering_Layer_(ATL)
// Code owners - dp-pinfo-team@amazon.com

"use strict";

//Bootstrapping Jungo, if not already available
window.smash.require || eval(window.smash.bootstrap(null));

var easeOut = [0, 0, 0.2, 1];
var easeIn = [0.4, 0, 1, 1];
var easeInOut = [0.4, 0, 0.2, 1];
var device = {
	os: "",
	version: "",
	delimiter: ""
};
var minVersion = {
	android: "4.0",
	iphone: "9_0"
};

function isNative(viewType) {
	if (viewType && viewType.toLowerCase() === "native") {
		return true;
	}
	return false;
}

function logJSError(exception, additionalInfo) {
	if (window.ueLogError) {
		window.ueLogError(exception, additionalInfo);
	}
}

function logCounter(sourceAnimationData, metricName, metricCount) {
	if (sourceAnimationData && sourceAnimationData.clientID) {
		metricName = metricName + "_" + sourceAnimationData.clientID;
	}
	if (!isNative(sourceAnimationData.viewType)) { // CSM
		if (typeof metricName !== "undefined") {
			var ue = window.ue;
			if (ue) {
				if (typeof metricCount !== "undefined") {
					ue.count(metricName, metricCount);
				} else {
					var existingValue = ue.count(metricName) || 0;
					ue.count(metricName, existingValue + 1);
				}
			}
		}
	} else { // DCM
		window.smash.require('smash/metric').then(function(metrics) {
			metrics.send({
				service: 'IESourcePageATL',
				value: metricName
			});
		});
	}
}

function deviceType(userAgent) {
	if (userAgent.toLowerCase().indexOf("android") >= 0) {
		return "android";
	} else if (userAgent.toLowerCase().indexOf("iphone") >= 0) {
		return "iphone";
	}
	return "";
}

function populateDeviceDetails() {
	var userAgent = navigator.userAgent;
	if (deviceType(userAgent) === "android") {
		var androidVersion = userAgent.slice(userAgent.toLowerCase().indexOf("android") + 8);
		androidVersion = androidVersion.substr(0, androidVersion.indexOf('/'));
		device["os"] = "android";
		device["version"] = androidVersion;
		device["delimiter"] = ".";
	} else if (deviceType(userAgent) === "iphone") {
		var iPhoneVersion = userAgent.slice(userAgent.toLowerCase().indexOf("os") + 3);
		iPhoneVersion = iPhoneVersion.substr(0, iPhoneVersion.indexOf(" "));
		device["os"] = "iphone";
		device["version"] = iPhoneVersion;
		device["delimiter"] = "_";
	}
}

function isPartsEmpty(parts) {
	return (parts === undefined || parts === "") ? true : false;
}

function versionCompare(ver1, ver2, delimiter) {
	var parts1 = ver1.split(delimiter);
	var parts2 = ver2.split(delimiter);
	var i = 0;
	while (i <= 20) { //just putting a hard limit to avoid infinite loop
		if (isPartsEmpty(parts1[i]) && isPartsEmpty(parts2[i])) {
			return 0;
		} else if (isPartsEmpty(parts1[i])) {
			return -1;
		} else if (isPartsEmpty(parts2[i])) {
			return 1;
		} else if (parseInt(parts1[i], 10) !== parseInt(parts2[i], 10)) {
			return parseInt(parts1[i], 10) > parseInt(parts2[i], 10) ? 1 : -1;
		}
		i++;
	}
	return 0;
}

function isDeviceBlacklisted(blacklist, version, delimiter) {
	var start = "",
		end = "";
	for (var i = 0; i < blacklist.length; i++) {
		if (blacklist[i].length > 0) {
			start = blacklist[i][0];
			end = blacklist[i][blacklist[i].length - 1];
			if (versionCompare(version, start, delimiter) >= 0 && versionCompare(version, end, delimiter) <= 0) {
				return true;
			}
		}
	}
	return false;
}

function isDeviceCompatible(blacklistedDevices) {
	if (device["os"] === "" || device["version"] === "" || device["delimiter"] === "") {
		return false;
	}
	if (versionCompare(device["version"], minVersion[device["os"]], device["delimiter"]) >= 0) {
		if (!blacklistedDevices || !blacklistedDevices[device["os"]]) {
			return true;
		}
		return !isDeviceBlacklisted(blacklistedDevices[device["os"]], device["version"], device["delimiter"]);
	}
	return false;
}

function getImageStyleTag(imageUrl) {
	var urlParts = imageUrl.split(".");
	var imageStyleTag;
	if (urlParts[urlParts.length - 2]) {
		imageStyleTag = urlParts[urlParts.length - 2];
		if (imageStyleTag.substring(0, 1) !== "_") {
			// no image style tag present
			imageStyleTag = undefined;
		}
	}
	return imageStyleTag;
}

function imageStyleTagCheck(imgStyle, imgUrl, viewType) {
	var whiteListTags = ["QL", "SX", "SY", "SL", "UL", "AC", "UX", "UY", "FMwebp"];
	if (!imgStyle) {
		imgStyle = getImageStyleTag(imgUrl);
		if (!imgStyle) {
			return {
				status: true,
				message: ""
			}
		}
	}
	var substrings = imgStyle.split('_');
	var individualTagCheck;
	for (var i = 0; i < substrings.length; i++) {
		if (substrings[i] !== "") {
			individualTagCheck = false;
			for (var j = 0; j < whiteListTags.length; j++) {
				if (substrings[i].indexOf(whiteListTags[j]) >= 0) {
					individualTagCheck = true;
					break;
				}
			}
			if (!individualTagCheck) {
				var message = "Style tag " + substrings[i] + " not whitelisted.";
				return {
					status: false,
					message: message
				}
			}
		}
	}
	return {
		status: true,
		message: ""
	}
}

function isValidNumber(num) {
	if (typeof(num) !== "number" || isNaN(num) || !isFinite(num)) {
		return false;
	}
	return true;
}

function isDetailPageURL(url) {
	//Need to figure out other alternatives for this
	var patterns = ["/dp/", "/gp/product/", "/gp/aw/d/"];
	url = url.replace(/([^:]\/)\/+/g, "$1"); // remove duplicate forward slashes
	for (var i = 0; i < patterns.length; i++) {
		if (url.indexOf(patterns[i]) > -1) {
			return true;
		}
	}
	return false;
}

function sanityCheck(sourceAnimationData, renderApiData) {
	var status = false,
		message = "";
	if (!sourceAnimationData) {
		message = "Unable to perform sky animation as sourceAnimationData is NULL/undefined";
		logCounter({
			viewType: "native"
		}, "ieatlsource_sourceAnimationDataNull");
		logCounter({
			viewType: "web-view"
		}, "ieatlsource_sourceAnimationDataNull");
	} else if (!renderApiData || !renderApiData.imageBlock.mainImage["v2.0"]) {
		message = "Unable to perform sky animation as ASIN blacklisted for transition by DPX";
		logCounter(sourceAnimationData, "ieatlsource_renderApiDataNull");
	} else if (!isNative(sourceAnimationData.viewType) && !isDeviceCompatible(renderApiData.imageBlock.mainImage["v2.0"].blacklistedDevices)) {
		message = "Device incompatible for IE, userAgent: " + navigator.userAgent;
		logCounter(sourceAnimationData, "ieatlsource_deviceIncompatible");
	} else if (!isDetailPageURL(sourceAnimationData.destinationPageURL)) {
		message = "Not a detail page URL: " + sourceAnimationData.destinationPageURL;
		logCounter(sourceAnimationData, "ieatlsource_notDetailPageUrl");
	} else if (sourceAnimationData.image.url.indexOf("no-img") > -1) { // no-image
		message = "URL " + sourceAnimationData.image.url + " represents a no-image";
		logCounter(sourceAnimationData, "ieatlsource_no-img");
	} else { // image style
		var imgStyle = sourceAnimationData.image.imageStyle;
		var imgUrl = sourceAnimationData.image.url;
		var styleCheck = imageStyleTagCheck(imgStyle, imgUrl, sourceAnimationData.viewType);
		if (styleCheck.status) {
			status = true;
		} else {
			message = styleCheck.message;
			logCounter(sourceAnimationData, "ieatlsource_styleTagNotWhitelisted");
		}
	}
	return {
		status: status,
		message: message
	}
}

function computeTargetImageCenter(imageDimension, destinationTopCoordinate, viewPortWidth) {
	var centerY = (imageDimension.height / 2) + destinationTopCoordinate;
	var centerX = viewPortWidth / 2;

	return {
		X: centerX,
		Y: centerY
	};
}

function computeDestinationPageTop(renderHintTop, imageHeight, imageBlockContainerHeight) {
	if (imageHeight < imageBlockContainerHeight) {
		renderHintTop = (renderHintTop + ((imageBlockContainerHeight - imageHeight) / 2));
	}
	return renderHintTop;
}

function computeParser(currValue, compute, base) {
	var newValue = currValue;
	for (var i = 0; i < compute.length; i++) {
		if (compute[i].base && compute[i].base !== base) {
			throw new Error("renderApiData contains " + compute[i].base + " parameters for " + base);
		}
		if (compute[i].operation) {
			switch (compute[i].operation) {
				case "multiply":
					newValue = newValue * (compute[i].value / 10000);
					break;
				case "add":
					newValue = newValue + compute[i].value;
					break;
			}
		}
	}
	return newValue;
}

function getImageBlockContainerWidth(containerWidth, viewPortWidth) {
	if (typeof(containerWidth) === "number") {
		return containerWidth;
	}
	var minWidth, compute, computedWidth;
	if (containerWidth.min) {
		minWidth = containerWidth.min[0];
		for (var i = 1; i < containerWidth.min.length; i++) {
			compute = containerWidth.min[i].compute;
			computedWidth = computeParser(viewPortWidth, compute, "vw");
			if (!isValidNumber(computedWidth) || computedWidth <= 0) {
				throw new Error("Invalid image block container width: " + computedWidth);
			}
			minWidth = minWidth > computedWidth ? computedWidth : minWidth;
		}
		return minWidth;
	} else {
		compute = containerWidth.compute;
		computedWidth = computeParser(viewPortWidth, compute, "vw");
		if (!isValidNumber(computedWidth) || computedWidth <= 0) {
			throw new Error("Invalid image block container width: " + computedWidth);
		}
		return computedWidth;
	}
}

function computeImageDimension(renderApiData, viewPortWidth, imageBlockContainerHeight, imageBlockContainerWidth, sourceImageHeight, sourceImageWidth) {
	var imageHeight, imageWidth;
	//Strategy to handle edge case of when image is neighther height bound nor width bound, is #TODO
	if (sourceImageWidth * (imageBlockContainerHeight / sourceImageHeight) >= imageBlockContainerWidth) {
		imageHeight = sourceImageHeight * (imageBlockContainerWidth / sourceImageWidth);
		imageWidth = imageBlockContainerWidth;
	} else {
		imageHeight = imageBlockContainerHeight;
		imageWidth = sourceImageWidth * (imageBlockContainerHeight / sourceImageHeight);
	}
	if (!isValidNumber(imageHeight) || !isValidNumber(imageWidth) || imageHeight <= 0 || imageWidth <= 0) {
		throw new Error("Invalid imageHeight: " + imageHeight + " or imageWidth: " + imageWidth);
	}
	return {
		height: imageHeight,
		width: imageWidth
	};
}



function getImageBlockContainerHeight(containerHeight, viewPortHeight) {
	if (typeof(containerHeight) === "number") {
		return containerHeight;
	}
	var minHeight = containerHeight.limit.min;
	var maxHeight = containerHeight.limit.max;
	var compute = containerHeight.limit.var.compute;
	var height = computeParser(viewPortHeight, compute, "vh");
	if (!isValidNumber(height) || height <= 0) {
		throw new Error("Invalid image block container height: " + height);
	}
	if (minHeight && height < minHeight) {
		height = minHeight;
	}
	if (maxHeight && height > maxHeight) {
		height = maxHeight;
	}
	return height;
}



function urlBuilder(url, key, value) {
	var newUrl;
	if (url.indexOf('?') > -1) {
		newUrl = url + "&" + key + "=" + value;
	} else {
		newUrl = url + "?" + key + "=" + value;
	}
	return newUrl;
}

function prepareDestinationPageUrl(sourceAnimationData, imageDimension, imageBlockContainerHeight, imageBlockContainerWidth) {
	var imageUrl = sourceAnimationData.image.url;
	var destPageUrl = sourceAnimationData.destinationPageURL;
	var urlParts = imageUrl.split(".");
	var imagePhysicalId, imageStyleTag;
	var physicalIdPosition = 3;
	imageStyleTag = getImageStyleTag(imageUrl);
	if (!imageStyleTag) {
		physicalIdPosition = 2;
	}
	if (sourceAnimationData.image.physicalId) {
		imagePhysicalId = sourceAnimationData.image.physicalId;
	} else if (urlParts[urlParts.length - physicalIdPosition]) {
		urlParts = urlParts[urlParts.length - physicalIdPosition].split("/");
		if (urlParts[urlParts.length - 1]) {
			imagePhysicalId = urlParts[urlParts.length - 1];
		}
	}
	if (imagePhysicalId) {
		destPageUrl = urlBuilder(destPageUrl, "dpID", imagePhysicalId);
		destPageUrl = urlBuilder(destPageUrl, "preST", imageStyleTag ? encodeURIComponent(imageStyleTag) : "");
		destPageUrl = urlBuilder(destPageUrl, "dpSrc", sourceAnimationData.clientID);
		destPageUrl = urlBuilder(destPageUrl, "ieImgH", Math.round(imageDimension.height));
		destPageUrl = urlBuilder(destPageUrl, "ieImgW", Math.round(imageDimension.width));
		destPageUrl = urlBuilder(destPageUrl, "ieImgCW", Math.round(imageBlockContainerWidth));
		destPageUrl = urlBuilder(destPageUrl, "ieImgCH", Math.round(imageBlockContainerHeight));
		destPageUrl = urlBuilder(destPageUrl, "isIR", isNative(sourceAnimationData.viewType) ? "0" : "1");
		//destPageUrl += "&dpID=" + imagePhysicalId + "&preST=" + (imageStyleTag? encodeURIComponent(imageStyleTag):"") + "&dpSrc=" + sourceAnimationData.clientID;
	} else {
		throw new Error('imagePhysicalId null/undefined!');
	}
	return destPageUrl;
}



function getDuration() {
	return 600;
}

function getFadeDuration() {
	return 300;
}

function getImageTranslateElement(image, targetImageCoordinates) {
	return {
		type: "translate",
		target: image.id,
		duration: getDuration(),
		easing: easeInOut,
		path: ["arcTo", 90, 0, 0, targetImageCoordinates.X, targetImageCoordinates.Y]
	};
}

function getImageScaleElement(img, imageDimension) {

	return {
		type: "scale",
		target: img.id,
		duration: getDuration(),
		easing: easeInOut,
		endWidth: imageDimension.width,
		endHeight: imageDimension.height
	};
}

function getScale(imageDimension, sourceAnimationData) {
	var xScale = (imageDimension.width / sourceAnimationData.imageDimension.width);
	var yScale = (imageDimension.height / sourceAnimationData.imageDimension.height);
	if (isValidNumber(xScale) && isValidNumber(yScale)) {
		return {
			X: xScale,
			Y: yScale
		}
	} else {
		throw new Error("Invalid xScale: " + xScale + " or yScale: " + yScale);
	}
}



function mashAnimationControl(targetImageCoordinates, sourceAnimationData, imageDimension, isNativePageTransition, destinationPageUrl) {
	return new window.Promise(function(resolve, reject) {
		window.smash.requireAll('smash/navigate', 'smash/View', 'smash/WebView', 'smash/ImageView',
			'smash/TransitionManager').spread(function(navigate, View, WebView, ImageView, TransitionManager) {


			var openPromise = navigate.open(destinationPageUrl);
			var imagePromise;

			if (!isNativePageTransition) {
				imagePromise = ImageView.create({
					tag: 'animatingImage',
					x: sourceAnimationData.imageStartCoordinates.left,
					y: sourceAnimationData.imageStartCoordinates.top,
					width: sourceAnimationData.imageDimension.width,
					height: sourceAnimationData.imageDimension.height,
					uri: sourceAnimationData.image.url,
					dataUri: sourceAnimationData.image.data
				});
			} else {
				imagePromise = View.fromId(sourceAnimationData.image.nativeImageToken);
			}

			window.Promise.spread([openPromise, imagePromise], function(openResult, img) {
				if (!openResult || !img) {
					reject("openResult or img null");
					return;
				}
				var currentPage = openResult.sourceView;
				var destPage = openResult.destinationView;
				var scale;

				try {
					scale = getScale(imageDimension, sourceAnimationData);
				} catch (err) {
					reject(err);
					return;
				}

				var x = imageDimension.width;
				var y = imageDimension.height;
				var y_0 = sourceAnimationData.imageStartCoordinates.top + sourceAnimationData.imageDimension.height / 2;
				var x_0 = sourceAnimationData.imageStartCoordinates.left + sourceAnimationData.imageDimension.width / 2;

				//FIX This, What is this ?
				var levelChange = 10;
				var srcPageScaleFactor = 1;
				if (levelChange > 0) {
					srcPageScaleFactor = scale;
				} else if (levelChange < 0) {
					srcPageScaleFactor = 1 / scale;
				} else {
					// TODO: Handle same-level transition
				}

				if (!img.id) {
					reject("image id null");
					return;
				}
				var t = new TransitionManager();
				t.addTarget(currentPage);
				t.addTarget(destPage);
				t.addTarget(img.id);

				t.addTransition({
					type: "sequence",
					elements: [{
						type: "fade",
						target: img.id,
						duration: 0,
						endOpacity: 1
					}, {
						type: "together",
						elements: [
							getImageTranslateElement(img, targetImageCoordinates),
							getImageScaleElement(img, imageDimension), {
								type: "translate",
								target: currentPage,
								duration: getDuration(),
								easing: easeInOut,
								path: ["arcTo", 90, 0, 0,
									x + srcPageScaleFactor.X * (sourceAnimationData.viewPort.width / 2 - x_0), y + srcPageScaleFactor.Y * (sourceAnimationData.viewPort.height / 2 - y_0)
								]
							}, {
								type: "scale",
								target: currentPage,
								easing: easeInOut,
								duration: getDuration(),
								endWidth: sourceAnimationData.viewPort.width * srcPageScaleFactor.X,
								endHeight: sourceAnimationData.viewPort.height * srcPageScaleFactor.Y
							}, {
								type: "fade",
								target: currentPage,
								duration: getFadeDuration(),
								easing: easeOut,
								startOpacity: 1,
								endOpacity: 0
							}
						]
					}]
				});
				t.execute().then(function(msg) {
					resolve("Animation Started.");
				}).catch(function(msg) {
					reject("Error in the Animation control");
				});
			});
		});
	});
}

var atlTriggerClass = {
	sourceAnimationTrigger: function(sourceAnimationData, renderApiData) {
		return new window.Promise(function(resolve, reject) {
			var weblabName = {
				android: "DETAIL_INSTANT_EXPERIENCE",
				iphone: "DETAIL_INSTANT_EXPERIENCE_IOS_76737"
			};
			var jsErrorInfo = {
				logLevel: "ERROR",
				attribution: "DetailPageIESourceATLAsset",
				message: "[IE][ATL][SourcePage]"
			};
			if (sourceAnimationData && sourceAnimationData.clientID) {
				jsErrorInfo.message += "[Client:" + sourceAnimationData.clientID + "]";
			}
			var promiseObject = {};

			if (!isNative(sourceAnimationData.viewType)) {
				populateDeviceDetails();
			}
			var prelimCheck = sanityCheck(sourceAnimationData, renderApiData);
			if (prelimCheck.status === false) {
				jsErrorInfo.message += prelimCheck.message;
				logJSError(jsErrorInfo);
				promiseObject.message = prelimCheck.message;
				promiseObject.destinationPageURL = undefined;
				if (sourceAnimationData) {
					if (sourceAnimationData.destinationPageURL) {
						promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T3");
					}
					if (sourceAnimationData.viewType) {
						logCounter(sourceAnimationData, "ieatlsource_nonAnimating_prelimCheckFailed");
						logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
					}
				}
				reject(promiseObject);
				return;
			}
			var imageBlockContainerHeight, imageBlockContainerWidth, imageDimension, destinationTopCoordinate;
			var targetImageCoordinates, isNativePageTransition = false,
				destinationPageUrl;
			try {
				imageBlockContainerHeight = getImageBlockContainerHeight(renderApiData.imageBlock.mainImage["v2.0"].containerHeight, sourceAnimationData.viewPort.height);
				imageBlockContainerWidth = getImageBlockContainerWidth(renderApiData.imageBlock.mainImage["v2.0"].containerWidth, sourceAnimationData.viewPort.width);
				imageDimension = computeImageDimension(renderApiData, sourceAnimationData.viewPort.width, imageBlockContainerHeight, imageBlockContainerWidth, sourceAnimationData.imageDimension.height, sourceAnimationData.imageDimension.width);
				destinationTopCoordinate = computeDestinationPageTop(renderApiData.imageBlock.mainImage["v2.0"].imagePosition.top.absolute, imageDimension.height, imageBlockContainerHeight);
				targetImageCoordinates = computeTargetImageCenter(imageDimension, destinationTopCoordinate, sourceAnimationData.viewPort.width);
				destinationPageUrl = prepareDestinationPageUrl(sourceAnimationData, imageDimension, imageBlockContainerHeight, imageBlockContainerWidth);
			} catch (err) {
				var message = "Exception encountered: " + err;
				jsErrorInfo.message += message;
				logJSError(jsErrorInfo);
				logCounter(sourceAnimationData, "ieatlsource_nonAnimating_paramComputationFailed");
				logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
				promiseObject.message = err;
				promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T3");
				reject(promiseObject);
				return;
			}

			if (isNative(sourceAnimationData.viewType)) {
				isNativePageTransition = true;
			}

			if (isNativePageTransition) {
				mashAnimationControl(targetImageCoordinates, sourceAnimationData, imageDimension, isNativePageTransition, destinationPageUrl).then(
					function(message) {
						logCounter(sourceAnimationData, "ieatlsource_animatingTransition");
						promiseObject.message = "Destination page sky animation started.";
						promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T2");
						resolve(promiseObject);
					}).catch(function(message) {
					var message = "MASH animation api call failed: " + message;
					jsErrorInfo.message += message;
					logJSError(jsErrorInfo);
					logCounter(sourceAnimationData, "ieatlsource_nonAnimating_mashCallFailed");
					logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
					promiseObject.message = message;
					promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T2");
					reject(promiseObject);
				});
			} else {
				window.smash.require('smash/weblab').then(function(weblab) {
					var weblabObj = weblab.get(weblabName[device["os"]]);
					weblabObj.getTreatmentAndRecordTrigger().then(function(weblabState) {
						if (weblabState.toUpperCase() !== "T1") {
							logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
							promiseObject.message = "[IE][ATL][SourcePage] Weblab " + weblabName[device["os"]] + " not in T1";
							promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T1");
							reject(promiseObject);
							return;
						} else {
							//MASH Animation api call
							mashAnimationControl(targetImageCoordinates, sourceAnimationData, imageDimension, isNativePageTransition, destinationPageUrl).then(
								function(message) {
									logCounter(sourceAnimationData, "ieatlsource_animatingTransition");
									promiseObject.message = "Destination page sky animation started.";
									promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T2");
									resolve(promiseObject);
								}).catch(function(message) {
								var message = "MASH animation api call failed: " + message;
								jsErrorInfo.message += message;
								logJSError(jsErrorInfo);
								logCounter(sourceAnimationData, "ieatlsource_nonAnimating_mashCallFailed");
								logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
								promiseObject.message = message;
								promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T2");
								reject(promiseObject);
							});
						}
					}).catch(function(message) {
						jsErrorInfo.message += "Weblab status rejected: " + message;
						logJSError(jsErrorInfo);
						logCounter(sourceAnimationData, "ieatlsource_nonAnimating_weblabCheckFailed");
						logCounter(sourceAnimationData, "ieatlsource_nonAnimatingTransition");
						promiseObject.message = message;
						promiseObject.destinationPageURL = urlBuilder(sourceAnimationData.destinationPageURL, "IEWS", "T3");
						reject(promiseObject);
						return;
					});
				});
			}
		});
	}
}

module.register({
	version: 1,
	sourceAnimationTrigger: atlTriggerClass.sourceAnimationTrigger
});